"""
"""
from __future__ import division, print_function, unicode_literals

from iirrational.testing import iirrational_data
import Pyro4
import json
import sys


if __name__ == "__main__":
    sys.excepthook = Pyro4.util.excepthook

    Pyro4.config.SERIALIZER = 'dill'
    with open('iirrational_pyro_con.json') as F:
        conf = json.load(F)
    uri = conf['v1']
    uri = uri.strip()
    factory = Pyro4.Proxy(uri)
    dat = iirrational_data('simple2')

    obj = factory.pyrosuper_call('rationalize', dat)

    factory.pyrosuper_call('annotate', obj)
    print(obj)




"""
"""
from __future__ import division, print_function, unicode_literals
import Pyro4
import threading
import contextlib

from . import meta_proxy


class MetaDaemon(object):

    def __init__(
        self,
        daemon = None,
        workers_N = 1,
    ):
        if daemon is None:
            daemon = Pyro4.Daemon()

        self.daemon = daemon
        self._worker_sem_value = None
        self.workers_N_set(workers_N)
        self.MTsafe = threading.Lock()

        #for caching GC
        self._metas_done = dict()

        #name -> object
        #to be wrapped and started in daemon
        self._objects = {}
        self._name_uri_reg = dict()

        #URI -> metaproxy(object) registry for json
        self._uri_registry = {}
        self._name_registry = {}
        return

    def workers_N_set(self, workers_N):
        #capture the remaining uses of the semaphore
        #if self._worker_sem_value is not None:
        #    for idx in range(self._worker_sem_value - 1):
        #        self.worker_sem.acquire()
        if workers_N is not None:
            self.worker_sem = threading.Semaphore(workers_N)
        else:
            #only used via the "with" interface, so this is OK
            @contextlib.contextmanager
            def fake_lock():
                yield
            fake_lock.acquire = lambda : None
            fake_lock.release = lambda : None
            self.worker_sem = fake_lock

    def register(
            self,
            obj    = None,
            name   = None,
            remove = False,
    ):
        if not remove:
            assert(obj is not None)
            if name is None:
                name = obj.__name__

            meta_object = meta_proxy.MetaProxy(self, obj, register = name)
            uri = str(meta_object.uri)

            self._objects[name] = obj
            self._uri_registry[uri] = meta_object
            self._name_registry[name] = meta_object
            self._name_uri_reg[name] = uri
        else:
            if name is None:
                assert(obj is not None)
                name = obj.__name__
            meta_obj = self._name_registry[name]
            obj_reg = self._objects[name]
            if obj is not None:
                assert(obj is obj_reg)

            del self._objects[name]
            del self._name_registry[name]
            del self._uri_registry[meta_obj.uri]
            del self._name_uri_reg[name]
            self.daemon.unregister(name)

    def uri2obj(self, uri):
        uri = str(uri)
        name, loc = uri.split('@')
        if not name.startswith('PYRO:'):
            raise RuntimeError("Proxy unwrapping can't recognize proxy")
        name = name[5:]
        obj = self.daemon.objectsById[name]
        return obj

    def workspace_dict(self):
        d = dict()
        for k, v in self._name_uri_reg.items():
            d[k] = str(v)
        return d

    def workspace_reset(self):
        #TODO
        temp = dict(self.daemon.objectsById)
        for k, v in temp.items():
            if k not in self._objects:
                self.daemon.unregister(k)
        return

    def workspace_gc(self):
        for k, v in self._metas_done.items():
            self.daemon.unregister(k)
        self._metas_done = dict()
        self.workspace_counts()
        for k, v in self._metas_done.items():
            self.daemon.unregister(k)
        self._metas_done = dict()
        return len(self.daemon.objectsById)

    def workspace_counts(self):
        #temp is for threadsafety
        temp = dict(self.daemon.objectsById)
        for k, v in temp.items():
            if isinstance(v, meta_proxy.MetaProxy):
                if not v.protect and v.done:
                    self._metas_done[k] = v
        return len(self._metas_done), len(self.temp)

    def workspace_close(self):
        self.daemon.shutdown()




"""
"""
from __future__ import division, print_function, unicode_literals
import Pyro4
import numbers
import sys

try:
    import numpy as np
except ImportError:
    import fake_numpy as np

if sys.version_info >= (3, 0):
    str_types = (str,)
else:
    str_types = (str, unicode)

class NeedsSubMeta(Exception):
    pass


@Pyro4.expose
class MetaProxy(object):
    #these are used for garbage collection
    protect = False
    done    = False

    def __init__(
            self,
            metaD,
            obj,
            register = True,
            protect  = None,
    ):
        self.metaD = metaD

        if isinstance(register, str_types):
            self.uri = self.metaD.daemon.register(self, register)
            if protect is None:
                protect = True
        elif register:
            #automatic registration!
            self.uri = self.metaD.daemon.register(self)

            if protect is None:
                protect = False
        else:
            self.uri = None
            if protect is None:
                protect = False

        #only assign in the rare event of True
        if protect:
            self.protect = protect

        self.obj = obj
        return

    def pyrometa_subsref(self, ref):
        #TODO
        raise NotImplementedError()

    def pyrometa_subsassgn(self, ref, val):
        #TODO
        raise NotImplementedError()

    def pyrometa_repr(self):
        return repr(self.obj)

    def pyrometa_str(self):
        return str(self.obj)

    def pyrometa_dir(self):
        return dir(self.obj)

    def pyrometa_getattr(self, name):
        checkname(self.obj, name)
        val = getattr(self.obj, name)
        return self.pyrometa_wrap(val)

    def pyrometa_setattr(self, name, val):
        checkname(self.obj, name)
        unval = self.pyrometa_unwrap(val)
        return setattr(self.obj, name, unval)

    def pyrometa_getitem(self, idx):
        idx = self.pyrometa_unwrap(idx)
        val = self.obj[idx]
        return self.pyrometa_wrap(val)

    def pyrometa_setitem(self, idx, val):
        idx = self.pyrometa_unwrap(idx)
        unval = self.pyrometa_unwrap(val)
        self.obj[idx] = unval
        return

    def pyrometa_call(self, name, *args, **kwargs):
        if name is None:
            val = self.obj
        else:
            checkname(self.obj, name)
            val = getattr(self.obj, name)
        unargs = self.pyrometa_unwrap(args)
        unkwargs = self.pyrometa_unwrap(kwargs)
        if check_MTsafe(val):
            with self.metaD.worker_sem:
                ret = val(*unargs, **unkwargs)
        else:
            with self.metaD.MTsafe:
                with self.metaD.worker_sem:
                    ret = val(*unargs, **unkwargs)
        return self.pyrometa_wrap(ret)

    @Pyro4.oneway
    def pyrometa_call_oneway(self, name, *args, **kwargs):
        if name is None:
            val = self.obj
        else:
            checkname(self.obj, name)
            val = getattr(self.obj, name)
        unargs = self.pyrometa_unwrap(args)
        unkwargs = self.pyrometa_unwrap(kwargs)
        if check_MTsafe(val):
            with self.metaD.worker_sem:
                ret = val(*unargs, **unkwargs)
        else:
            with self.metaD.MTsafe:
                with self.metaD.worker_sem:
                    ret = val(*unargs, **unkwargs)
        return

    def pyrometa_done(self):
        if not self.protect and self.done:
            raise RuntimeError("done set twice!")
        self.done = True

    def pyrometa_wrap(self, obj, throw = False):
        """
        throw causes an exception instead of creating a sub MetaProxy
        """
        if isinstance(obj, np.ndarray):
            return obj
        elif obj is None:
            return obj
        elif isinstance(obj, numbers.Number):
            return obj
        elif isinstance(obj, str_types):
            return obj
        elif isinstance(obj, list):
            try:
                sublist = [self.pyrometa_wrap(o, throw = True) for o in obj]
            except NeedsSubMeta:
                if throw:
                    raise
                else:
                    return MetaProxy(self.metaD, obj)
            else:
                return sublist
        elif isinstance(obj, tuple):
            try:
                subtup = tuple((self.pyrometa_wrap(o, throw = True) for o in obj))
            except NeedsSubMeta:
                if throw:
                    raise
                else:
                    return MetaProxy(self.metaD, obj)
            else:
                return subtup
        elif isinstance(obj, set):
            try:
                subset = set((self.pyrometa_wrap(o, throw = True) for o in obj))
            except NeedsSubMeta:
                if throw:
                    raise
                else:
                    return MetaProxy(self.metaD, obj)
            else:
                return subset
        elif isinstance(obj, dict):
            try:
                subdict = dict()
                for k, v in obj.items():
                    subdict[k] = self.pyrometa_wrap(v, throw = True)
            except NeedsSubMeta:
                if throw:
                    raise
                else:
                    return MetaProxy(self.metaD, obj)
            else:
                return subdict
        if throw:
            raise NeedsSubMeta()

        return MetaProxy(self.metaD, obj)

    def pyrometa_unwrap(self, obj):
        """
        throw causes an exception instead of creating a sub MetaProxy
        """
        if isinstance(obj, list):
            sublist = [self.pyrometa_unwrap(o) for o in obj]
            return sublist
        elif isinstance(obj, tuple):
            subtup = tuple((self.pyrometa_unwrap(o) for o in obj))
            return subtup
        elif isinstance(obj, set):
            subset = set((self.pyrometa_unwrap(o) for o in obj))
            return subset
        elif isinstance(obj, dict):
            subdict = dict()
            for k, v in obj.items():
                subdict[k] = self.pyrometa_unwrap(v)
            return subdict
        elif isinstance(obj, MetaProxy):
            return obj.obj
        elif isinstance(obj, Pyro4.Proxy):
            uri = obj._pyroUri
            obj = self.metaD.uri2obj(uri)
            if isinstance(obj, MetaProxy):
                return obj.obj
            else:
                return obj
        else:
            return obj


def checkname(obj, name):
    if name.startswith('_'):
        try:
            allow = obj._msurrogate_unsafe
        except AttributeError:
            allow = False
        if not allow:
            raise AttributeError('attributes/methods starting with _ not allowed')

def check_MTsafe(obj):
    """
    Recurse up object and parents via __self__ of instancemethod types in search of an obj._msurrogate_MT flag indicating to use full multithreading
    """
    #TODO, could do isinstance checks against types.FunctionType and types.MethodType, but those seem to carry __self__
    while True:
        try:
            safe = obj._msurrogate_MT
        except AttributeError:
            pass
        else:
            return safe

        try:
            parent = obj.__self__
        except AttributeError:
            return False
        obj = parent



"""
"""
from __future__ import division, print_function, unicode_literals

from os import path

from .version import (
    version,
    __version__
)

from .meta_app import (
    SurrogateApp,
    cookie_setup,
)

from .subproc_server import (
    SurrogateSubprocess,
)

def matlabpath():
    return path.abspath(path.split(__file__)[0])




"""
An msurrogate model server. Start this code to connect from other applications
"""
from __future__ import division, print_function, unicode_literals
import argparse
import socket
import Pyro4
import collections
import sys
import json
import uuid

from .meta_daemon import MetaDaemon

if sys.version_info > (3, 0):
    pass
else:
    input = raw_input


class SurrogateApp(object):

    def __init__(
        self,
        argparser = None,
    ):

        if argparser is None:
            argparser = argparse.ArgumentParser(
                description = __doc__
            )
        self.argparser = argparser

        self.forced = set()
        self.defaults = {
            'public'         :         False,
            'host'           :         None,
            'pubhost'        :         None,
            'pubport'        :         None,
            'port'           :         None,
            'cookiefile'     :         None,
            'secret'         :         None,
            'workers'        :         4,
            'multithread'    :         True,
            #non-config options
            'allow-dill'     :         True,
            'allow-pickle'   :         True,
            'workspace-auto' :         True,
            'multithreaded-auto-off' : False,
            'hide-secret' :            True,
            'serializer-preferred' :   None,
        }

        self.args        = None
        self.daemon      = None
        self.meta_daemon = None

    def option(self, name, value, force = False):
        if force:
            self.forced.add(name)
        else:
            try:
                self.forced.remove(name)
            except KeyError:
                pass
        self.defaults[name] = value
        return

    def parse_args(self, args = None):
        """
        Setup the remainder of the argparser and then parse the arguments
        """

        #public
        #use this if it should ever bind off of localhost (must be explicit)
        if 'public' not in self.forced:
            self.argparser.add_argument(
                '--public', dest = 'public', default = self.defaults['public'],
                action = 'store_true',
                help = 'Allow binding on exposed interfaces (required for host/pubhost to work)'
            )

            if 'host' not in self.forced:
                #host
                #host
                self.argparser.add_argument(
                    '-n', '--host', dest = 'host', default = self.defaults['host'],
                    help = (
                        'Hostname to bind to. Defaults to localhost unless public is specified, '
                        'in which case the default is the current hostname. '
                        'May be "*" to bind all interfaces, but pubhost should also be provided'
                    ),
                )

            #pubhost
            if 'pubhost' not in self.forced:
                self.argparser.add_argument(
                    '-N', '--public-host', dest = 'pubhost', default = self.defaults['pubhost'],
                    help = (
                        'Hostname to provide for URIs '
                        'defaults to the same as host, unless host is "*" and pubhost should be provided'
                    ),
                )
            #host name through NAT

        #port
        #default (None) is a random port
        if 'port' not in self.forced:
            self.argparser.add_argument(
                '-p, --port', dest = 'port', default = self.defaults['port'],
                type = int,
                help = (
                    'port to listen on. Defaults to a random unused port'
                ),
            )

        #pubport
        #port forearded through NAT
        if 'pubport' not in self.forced:
            self.argparser.add_argument(
                '-P', '--public-port', dest = 'pubport', default = self.defaults['pubport'],
                type = int,
                help = (
                    'port to provide in URIs. Should be forwarded through a NAT.'
                ),
            )

        #secret
        #secret to use for connection start
        #if it is '-' then it will be specified through stdin
        if 'secret' not in self.forced:
            self.argparser.add_argument(
                '-S', '--secret', dest = 'secret', default = self.defaults['secret'],
                help = (
                    'Secret Phrase for slight security. Default is random. May be "-" to read from stdin. '
                    'May be "" to not use a secret'
                ),
            )

        #cookiefile
        #if it is '-' it will be output to stdout
        #if it is '' there will be NO output
        if 'cookiefile' not in self.forced:
            self.argparser.add_argument(
                '-c', '--cookie-file', dest = 'cookiefile', default = self.defaults['cookiefile'],
                help = (
                    'cookie file to write with interface URIs and connection information. '
                    'May be "-" to supply on stdout. May be "" to write no file at all.'
                ),
            )

        #multithread
        #if true, specify and use it in Pyro4
        #if false, use single threaded server
        #if None, don't specify (default is threaded), presumably the user wants to override
        def_MT = self.defaults['multithread']
        if 'multithread' not in self.forced:
            if def_MT is None:
                #don't specify, user is doing it
                pass
            elif def_MT:
                self.argparser.add_argument(
                    '-M, --no-multithread', dest = 'multithread', default = def_MT,
                    action = 'store_false',
                    help = (
                        'Disable multithreaded operation. Workers is effectively one. Use if OS is running out of threads during connection.'
                    ),
                )
            else:
                self.argparser.add_argument(
                    '-m, --with-multithread', dest = 'multithread', default = def_MT,
                    action = 'store_true',
                    help = (
                        'Enable multithreaded operation. Workers is then used.'
                    ),
                )

        if 'workers' not in self.forced:
            #workers
            #number of workers to bound with the semaphore, of multithread is forced false, it wont be asked
            self.argparser.add_argument(
                '-W', '--workers', dest = 'workers', default = None,
                help = (
                    'Workers limit for computation. Limits the number of simultaneously running threads. Each connection still gets a thread.'
                ),
            )

        self.args = self.argparser.parse_args(args)

        for k, v in self.defaults.items():
            if k not in self.args:
                self.args.__dict__[k] = v

        #now check!
        self.multithread = self.args.multithread
        if self.args.multithread is not None:
            if self.args.multithread:
                if self.args.workers is None:
                    self.workers = self.defaults['workers']
                else:
                    self.workers = self.args.workers
                if self.workers < 1:
                    raise RuntimeError("Number of Workers must be greater than 0")
                if self.workers == 1 and self.defaults['multithreaded-auto-off']:
                    self.multithread = False
            else:
                if self.args.workers is not None:
                    raise RuntimeError("Cannot Specify Workers if not running multithreaded")
                self.workers = None

        self.public  = self.args.public
        self.host    = self.args.host
        self.pubhost = self.args.pubhost
        self.port    = self.args.port
        if self.port is None:
            self.port = 0
        self.pubport = self.args.pubport

        if not self.public:
            if self.args.host is not None:
                raise RuntimeError("Cannot Specify Host if not running public")
            self.host = 'localhost'
        else:
            if self.host is None:
                self.host = socket.gethostname()

        self.secret = self.args.secret
        self.cookiefile = self.args.cookiefile
        if self.cookiefile is None:
            raise RuntimeError("Must Specify a cookiefile to output")

        if self.multithread is None:
            pass
        elif self.multithread:
            Pyro4.config.SERVERTYPE = 'thread'
        else:
            Pyro4.config.SERVERTYPE = 'multiplex'

        if self.secret == '-':
            self.secret = input()
            if self.defaults['hide-secret']:
                self.hide_secret = True
            else:
                self.hide_secret = False
        elif self.secret is None:
            self.secret = str(uuid.uuid4())
            self.hide_secret = False
        else:
            self.hide_secret = False

        if self.secret is not '':
            Pyro4.Daemon._pyroHmacKey = bytes(self.secret)
            Pyro4.Proxy._pyroHmacKey = bytes(self.secret)

        Pyro4.config.SERIALIZERS_ACCEPTED = set(Pyro4.config.SERIALIZERS_ACCEPTED)
        if self.defaults['allow-dill']:
            Pyro4.config.SERIALIZERS_ACCEPTED.add('dill')
        if self.defaults['allow-pickle']:
            Pyro4.config.SERIALIZERS_ACCEPTED.add('pickle')
        return

    def daemon_setup(self, daemon = None):
        if self.args is None:
            self.parse_args()

        if self.daemon is None:
            if daemon is None:
                self.daemon = Pyro4.Daemon(
                    host    = self.host,
                    port    = self.port,
                    nathost = self.pubhost,
                    natport = self.pubport,
                )
            else:
                self.daemon = daemon
        return

    def meta_daemon_setup(self, meta_daemon = None):
        if self.meta_daemon is None:
            if self.daemon is None:
                self.daemon_setup()

            if meta_daemon is None:
                self.meta_daemon = MetaDaemon(
                    daemon = self.daemon,
                    workers_N = self.workers,
                )
            else:
                self.meta_daemon = meta_daemon

        if self.defaults['workspace-auto']:
            self.meta_daemon.register(self.meta_daemon.workspace_dict)
            self.meta_daemon.register(self.meta_daemon.workspace_reset)
            self.meta_daemon.register(self.meta_daemon.workspace_counts)
            self.meta_daemon.register(self.meta_daemon.workspace_gc)
            self.meta_daemon.register(self.meta_daemon.workspace_close)

        return self.meta_daemon

    def cookie_dict(self):
        d = dict(
            workspace = self.meta_daemon.workspace_dict(),
            host      = self.host,
        )
        if not self.hide_secret:
            d['secret'] = self.secret
        serializer_pref = self.defaults['serializer-preferred']
        if serializer_pref is not None:
            d['serializer-preferred'] = serializer_pref
        return d

    def cookie_write(self):
        if self.meta_daemon is None:
            self.meta_daemon_setup()

        if self.cookiefile == '':
            #no cookie
            pass
        elif self.cookiefile == '-':
            #guarantee a newline before COOKIE_START and AFTER COOKIE_END
            print('')
            print('COOKIE_START')
            json.dump(
                self.cookie_dict(), sys.stdout,
                indent = 4,
                sort_keys = True,
            )
            print()
            print('COOKIE_END')
            sys.stdout.flush()
        else:
            with open(self.cookiefile, 'w') as F:
                json.dump(
                    self.cookie_dict(), F,
                    indent = 4,
                    sort_keys = True,
                )

    def run_loop(self):
        if self.meta_daemon is None:
            self.meta_daemon_setup()

        self.cookie_write()
        self.meta_daemon.daemon.requestLoop()
        return


def cookie_setup(cookie_dict):
    if not isinstance(cookie_dict, collections.Mapping):
        #then it must be a string
        cookie_dict = json.loads(cookie_dict)

    secret = cookie_dict.get('secret', '')

    if secret is not None and secret != '':
        Pyro4.Daemon._pyroHmacKey = bytes(secret)
        Pyro4.Proxy._pyroHmacKey  = bytes(secret)

    serializer = cookie_dict.get('serializer-preferred', None)

    if serializer is not None:
        Pyro4.config.SERIALIZER = serializer

    sys.excepthook = Pyro4.util.excepthook
    return cookie_dict['workspace']









"""
"""
from __future__ import division, print_function, unicode_literals
from msurrogate import SurrogateApp


def test_ping(text = ''):
    print("PONG" + text)


if __name__ == "__main__":
    app = SurrogateApp()
    app.option('serializer-preferred', 'dill')
    metaD = app.meta_daemon_setup()
    metaD.register(test_ping)
    app.run_loop()




"""
An msurrogate model server. Start this code to connect from other applications
"""
from __future__ import division, print_function, unicode_literals
import sys
import os
import json
import uuid
import subprocess
import threading


class SurrogateSubprocess(object):

    def __init__(
        self,
        python_call    = None,
        module_name    = None,
        env            = None,
        stdout_prepend = '  PYOUT:',
        stderr_prepend = '  PYERR:',
        args           = [],
        relay_stdout   = True,
        relay_stderr   = True,
    ):

        self.stdout_prepend = stdout_prepend
        self.stderr_prepend = stderr_prepend
        self.relay_stdout = relay_stdout
        self.relay_stderr = relay_stderr

        #update the environment and clear variables with None
        if env is not None:
            env_use = dict(os.environ)
            for k, v in env.items():
                if v is None:
                    try:
                        del env_use[k]
                    except KeyError:
                        pass
                else:
                    env_use[k] = v

        if python_call is None:
            python_call = sys.executable

        if module_name is None:
            raise RuntimeError("Must specify module_name")

        #the -u is for unbuffered output (otherwise code must use flush everywhere)
        #-S - and -c - are for the secret to be relayed via stdin and the cookie via stdout.
        self.proc = subprocess.Popen(
            [python_call, '-u', '-m', module_name, '-S', '-', '-c', '-'] + args,
            stdout = subprocess.PIPE,
            stdin  = subprocess.PIPE,
            stderr = subprocess.PIPE,
            env    = env_use,
            bufsize=1,
        )

        #write the secret
        mysecret = str(uuid.uuid4())
        self.proc.stdin.write(mysecret + "\n")

        #start the error feed first so that we can see what is going on
        self.stderr_thread = threading.Thread(
            target = self._stderr_thread_loop,
            name = 'python subprocess stderr feed'
        )
        self.stderr_thread.daemon = True
        self.stderr_thread.start()

        while True:
            line = self.proc.stdout.readline()
            if not line:
                raise RuntimeError("Subprocess Did not complete its output")
            if line.strip() == 'COOKIE_START':
                break
            sys.stdout.write(self.stdout_prepend + line)

        cookie_lines = []
        while True:
            line = self.proc.stdout.readline()
            if not line:
                raise RuntimeError("Subprocess Did not complete its output")
            if line.strip() == 'COOKIE_END':
                break
            cookie_lines.append(line)
        self.cookie_dict = json.loads(''.join(cookie_lines))

        #add the secret back in
        if 'secret' not in self.cookie_dict:
            self.cookie_dict['secret'] = mysecret

        self.stdout_thread = threading.Thread(
            target = self._stdout_thread_loop,
            name = 'python subprocess stdout feed'
        )
        self.stdout_thread.daemon = True
        self.stdout_thread.start()

        return

    def stop(self):
        proc = self.proc
        if proc is not None:
            #TODO need try-catch if already stopped?
            proc.kill()
            #TODO check if process really terminated
            #stops the threads
            self.proc = None
        return

    def _stdout_thread_loop(self):
        proc = self.proc
        while proc is not None:
            line = proc.stdout.readline()
            if not line:
                break
            if self.relay_stdout:
                sys.stdout.write(self.stdout_prepend + line)
            proc = self.proc

    def _stderr_thread_loop(self):
        proc = self.proc
        while proc is not None:
            line = proc.stderr.readline()
            if not line:
                break
            if self.relay_stderr:
                sys.stderr.write(self.stderr_prepend + line)
            proc = self.proc






"""
"""
from __future__ import division, print_function, unicode_literals

from msurrogate.subproc_server import ServerSubprocess


if __name__ == "__main__":
    app = ServerSubprocess(
        module_name = 'msurrogate.ping_test',
    )
    print("SECRET: ", app.mysecret)
    print(app.cookie_dict)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import division, print_function, unicode_literals
import os
import sys
from os import path
from setuptools import find_packages, setup
from distutils.command.bdist import bdist
from distutils.command.sdist import sdist

version = '0.9.0.rc1'

def check_versions():
    print('versions checked')
    try:
        curpath = path.abspath(path.realpath(os.getcwd()))
        setuppath = path.abspath(path.realpath(path.split(__file__)[0]))

        if curpath != setuppath:
            sys.path.append(setuppath)
        import msurrogate

        modfile = path.abspath(path.realpath(path.split(msurrogate.__file__)[0]))
        mod_relpath = path.relpath(modfile, setuppath)
        if mod_relpath == 'msurrogate':
            if msurrogate.__version__ != version:
                print("WARNING: Stated module version different than setup.py version", file = sys.stderr)
                print("         '{0}' != '{1}'".format(msurrogate.__version__, version), file = sys.stderr)
                print("Fix version.py and setup.py for consistency")

        import subprocess
        try:
            git_tag = subprocess.check_output(['git', 'describe', '--tags'])
            git_tag = git_tag.strip()
            git_tag = str(git_tag)
        except subprocess.CalledProcessError:
            pass
        else:
            if git_tag != version:
                if git_tag.startswith(version):
                    print("WARNING: latex git-tag has commits since versioning", file = sys.stderr)
                    print("         '{0}' ---> '{1}'".format(version, git_tag), file=sys.stderr)
                else:
                    print("WARNING: latex git-tag different than setup.py version", file = sys.stderr)
                    print("         '{0}' != '{1}'".format(version, git_tag), file=sys.stderr)
                print("         Perhaps update versions in setup.py and msurrogate.version, git commit, then git tag")
                print("         otherwise fix tag if not yet git pushed to remote (see DISTRIBUTION-README.md)")

    except ImportError:
        pass


cmdclass = dict()

class check_bdist(bdist):
    def run(self):
        bdist.run(self)
        check_versions()

class check_sdist(sdist):
    def run(self):
        sdist.run(self)
        check_versions()

cmdclass['bdist'] = check_bdist
cmdclass['sdist'] = check_sdist

try:
    from wheel.bdist_wheel import bdist_wheel
    class check_bdist_wheel(bdist_wheel):
        def run(self):
            bdist_wheel.run(self)
            check_versions()
except ImportError:
    pass

cmdclass['bdist_wheel'] = check_bdist_wheel


if __name__ == "__main__":
    setup(
        name         = 'msurrogate',
        version      = version,
        url          = 'https://github.com/mccullerlp/msurrogate',
        author       = 'Lee McCuller',
        author_email = 'Lee.McCuller@gmail.com',
        license      = 'Apache v2',
        description  = (
            'Interface With an out-of-process (possibly remote) python instance from Matlab'
        ),
        packages=find_packages(
            exclude=['docs'],
        ),
        install_requires = [
            'numpy',
            'Pyro4',
        ],
        cmdclass = cmdclass,
        zip_safe = False,
        keywords = ['Matlab', 'IPC', 'Pyro4',],
        classifiers=[
            'Development Status :: 4 - Beta',
            'Intended Audience :: Developers',
            'Topic :: Scientific/Engineering',
            'License :: OSI Approved :: Apache Software License',
            'Operating System :: OS Independent',
            'Programming Language :: Python',
            'Programming Language :: Python :: 2',
            'Programming Language :: Python :: 2.7',
            'Programming Language :: Python :: 3',
            'Programming Language :: Python :: 3.4',
            'Programming Language :: Python :: 3.5',
            'Programming Language :: Python :: 3.6',
        ],
    )




"""
AUTOGENERATED by git post-commit hook. Note that these are always from the PREVIOUS commit, and so may be out of date by one log entry
"""
git_branch    = "master"
git_shorthash = "caf37cc"
git_longhash  = "caf37ccbb0791eea885c2044224e8d0fa2ddaa66"
git_last_tag  = "0.9.0.dev3"
